% binorat.m: Rattus binomialis
%
% MATLAB concepts covered:
% 1. 'rand'
% 2. simulations for answering questions of probability
% 3. 'for' loops
% 4. indexing arrays
% 5. 'tic' and 'toc' to time code execution
% 6. binomial distribution
% 7. pdf's and cdf's
%
% RTB wrote it, 23 April 2011 as "binodemo.m"
% RTB revised it, 23 May 2011 as "binorat.m"

% A rat is performing a "2-alternative forced choice" (2AFC) task in which 
% it must identify an odor presented at a central port. If it detects 
% odor 'A' it should choose the right-hand port for a reward; if it detects
% odor 'B' is should choose the other port.

% The rat in the movie clip is highly trained and gets it right almost
% every time. But if we go back in time to the early training period, we
% would find that the rat seems to get it wrong as often as he gets it
% right. So you decide to do a test and keep track of his correct rate for
% a block of 50 trials. After 50 trials, we see that the rat has gotten 
% 31 trials correct (19 trials wrong) for an average of 62%.
% Is the rat learning the task or might he still be guessing?

% Let's say that we know nothing about probability theory or the binomial
% distribution and just use the brute force power of a simple simulation.
% We can start by rephrasing the question. The null hypothesis is that the
% rat is guessing. Since there are only two possible answers, this corresponds
% to a correct probability of 0.5, which is identical to the rate at which 
% tossing a fair coin should produce a "tail." So our previous question can 
% be put as how often one would expect to get 31 (or more) tails out of 50
% coin tosses. Why is the parenthetical "or more" added to the previous sentence?
% Well, we're not really interested in the probability of the rat getting
% *exactly* 31 of 50 trials correct. We're looking for conditions under
% which we would *reject* the null hypothesis of guessing. So we would
% clearly count any performance that was better than 31 as well.

% To answer with a real coin would take a long time, but computers are fast, so
% we can let the computer do the flipping. How? The MATLAB 'rand' command
% produces random entries chosen from a uniform distribution on the
% interval (0.0,1.0), so a single coin toss can be simulated by picking a
% number from 'rand' and asking if it is, say, greater than 0.5. We can do
% this 50 times and see how many times we satisfy the inequality--this
% would constitute one simulation of our problem. However, to know how
% often a certain simulation result is obtained, we need to repeat the simulation
% many times. Then we can ask how often a given result (31 or more tails)
% happens.

%% Start by initializing some variables:
p = 0.5;        % our "null hypothesis" probability
n = 50;         % the number of coin tosses in one simulation
x = 31;         % the number of "tails" we obtained in real life
nsim = 10000;    % the number of simulations we're going to run

% We'll also need a variable to hold the results of our simulations:
tosses = zeros(n,1);        % results of individual coin tosses on one simulation
results = zeros(nsim,1);    % results of each simulation

% I've initialized the results vectors with zeros as if "failure" were the
% default. If we succeed (i.e. get a "tail"), we'll change the
% corresponding zero to a one.

% For most programmers, the most intuitive way to run the simulations would
% be as a pair of nested 'for' loops. The inner loop would consist of a
% single simulation of 50 coin tosses; the outer loop would be the number
% of simulations.
tic;
for k = 1:nsim
    
    % one round of 50 coin tosses
    for m = 1:n
        if rand > p, tosses(m) = 1; end         % a single coin flip
    end
    results(k) = sum(tosses);
    %if sum(tosses) >= x, results(k) = 1; end    % one simulation
    tosses = zeros(n,1);                        % need to re-set prior to next simulation
end

% That's all there is to it. Now we just ask how many times we "succeeded"
% in getting 20 or more tails. The probability is simply the number of
% times we succeeded divided by the number of times we tried.
prob = sum(results >= x) ./ nsim
toc
% We come up with a probability of something greater than 0.05. Every
% simulation will ge a little bit different--it is random, after all. But
% you can satisfy yourself that it gets close to the same answer each time
% by running the entire script multlple times. And we could make our
% variation in 'prob' arbitrarily small by running more and more
% simulations.

% So we've answered our question. We cannot reject the null hypothesis that
% the rat is still guessing. How many would he have to get right before we
% concluded that he probably was not guessing?

%% This is nice, but to a true MATLAB programmer, the presence of 'for'
% loops in the code always raises suspicions. Because MATLAB is a
% vector-based language, we can often do things much more efficiently using
% indexing and operators (such as '>') that work across entire arrays or
% functions that work along a specific dimension, such as down columns. For
% example, we don't need to call 'rand' so many times. In fact, we can call
% it only once and get all of the coin tosses in one giant array:
tic;
allTosses = rand(n,nsim);   % a 50 x nsim matrix of random numbers
L = allTosses > p;          % a logical array of 0s and 1s; result of all flips

% By default, 'sum' works down columns. But you can also specify which
% dimension will be used: sum(L) is the same as sum(L,1); sum(L,2) would
% sum along rows.
allResults = sum(L);
LL = allResults >= x;
prob = sum(LL) ./ nsim
toc

% This is much more satisfying code (fewer lines!), and it would execute
% much more quickly. You can test the speed using MATLAB's 'tic' and 'toc'
% functions. You execute a 'tic' at the beginning of your code, and then a
% 'toc' at the end, and you'll get the amount of elapsed time. Try it!

% When I do it for the 'for-loop' version, I get: Elapsed time is 0.139723
% seconds (with nsim = 1000). For the non-for-loop version, I get: Elapsed
% time is 0.023866 seconds. That's nearly 6 times faster for the 2nd
% version.

%% Of course, we can do even better by taking advantage of all of the
% probability distributions provided in the statistics toolbox. You might
% vaguely recall that events like coin tosses, where there are only two
% possible outcomes, are described by the binomial distribution. Remebering
% only this key word, 'binomial', we can quickly find all of the functions
% that are relevant with:
lookfor binomial

% This produces a list of about 40 functions along with a 1-sentence
% description of what each computes. If we wanted to know more about a
% given function, we would just type its name after the 'help' command:
help binopdf

%% This sounds useful. It gives us the probability of getting X successes
% out of N tries if the probability of success on each trial is P. What is
% the probability of getting (exactly) 5 heads from 10 tosses?
binopdf(5,10,0.5)

% What is the probability of getting 5 or fewer heads? We could call
% 'binopdf' 6 times in a 'for' loop, but this is MATLAB, so we might expect
% the function to do it for us.
y = binopdf(0:5,10,0.5);
sum(y)

%% Given that one often wants to know things like "probability of X or
% fewer" we might not be surprised to find a function that does this
% directly. This is called the "cumulative distribution function" and for
% binomial data, we would use 'binocdf'. What is the equivalent cdf command
% for the two lines of code above?
binocdf(5,10,0.5)

% One of the wonderful things about MATLAB's statistics toolbox is that it
% provides pdf's and cdf's for just about any statistical function you
% might imagine. Type:
lookfor 'cumulative distribution'

% Back to our rat question. We want to know the probability of getting
% 31 heads or more out of 50 tosses. We could use the pdf:
y = binopdf(31:50,50,0.5);
sum(y)

% But since we know that the sum of all possible outcomes (0 up to
% 50) must sum to one, we can calculate this as 1 - (probability of 30 or
% fewer).
prob = 1 - binocdf(30,50,0.5)

% Convince yourself that these are equivalent.
% Not bad. Just one line of code and much faster (0.001155 seconds).

% OK, so what do we think? Is the rat "getting it" or is he just guessing?
% How many would he need to get right (out of 50) before you believed?

% Exercise: Compute the 95% upper and lower confidence bounds for:
%   a) 30 correct out of 50
%   b) 60 of 100
%   c) 300 of 500